@@ -23,7 +23,7 @@ class StatsDump(object):
         s += "<h3>Global stats</h3>\n"
         s += stats_html_table(stats.get_stats())
         for spider, spider_stats in stats.iter_spider_stats():
-            s += "<h3>%s</h3>\n" % spider.domain_name
+            s += "<h3>%s</h3>\n" % spider.name
             s += stats_html_table(spider_stats)
         s += "</body>\n"
         s += "</html>\n"
